#ifdef WIN32
#pragma comment(lib, "SDL.lib")
#pragma comment(lib, "SDLmain.lib")
#endif

#include <iostream>
#include <cstdio>
#include <cstring>
#include <SDL.h>
#include <SDL_ttf.h>

#include "sprite_handler.h"
#include "fps_handler.h"

enum DIRECTION {LEFT, RIGHT, UP, DOWN};

void move_badguy(Sprite_Handler& handler);

int main(int argc, char **argv)
{

  SDL_Event event;

  bool exit = false;
  bool active = true;
  Uint8 *key_state;
  short delta_x = 0;
  short delta_y = 0;
  
  DIRECTION p_dir = LEFT;
  
  std::string temp_fps; //string used to transfer data from fps_handler to sprite_handler
  Sprite_Handler sprite_handler; //sprite handler that takes care of processing all sprites in
                                  //the game
  FPS_Handler fps_handler;      //class that's used to regulate frames per second.


  
  //the main loop
  while (!exit)
  {
    move_badguy(sprite_handler);  //bad guy doesn't care if you're paying attention.
  
    if (active) //if it's not minimized...
    {
      //draw scene and check for collision between what's moved.
      if (!sprite_handler.draw_scene(delta_x, delta_y)) //there's a collision
      {
        switch(p_dir)
	{
	  case LEFT:
	    sprite_handler.player_obj.x += 25;
	    delta_x = 25;
            delta_y = 0;
	    break;
	    
	  case RIGHT:
            sprite_handler.player_obj.x -= 25;
	    delta_x = -25;
            delta_y = 0;
	    break;
	    
	  case UP:
	    sprite_handler.player_obj.y += 25;
	    delta_x = 0;
            delta_y = 25;
	    break;
	    
	  case DOWN:
	    sprite_handler.player_obj.y -= 25;
	    delta_x = 0;
            delta_y = -25;
	    break;
	}
      }
    
      fps_handler.update_frames();                  //after every screen draw we
                                                    //increase the frame count.
      
      temp_fps = fps_handler.get_fps();            //sets current fps to string

      fps_handler.regulate_fps();                //waits if games running too fast to
      						// keep fps steady.

      if (temp_fps != "")                     //if get_fps didn't return an empty string
        sprite_handler.updateFPS(temp_fps);   //update the fps msg surface.


    }
  
    while (SDL_PollEvent(&event))
    {
      switch(event.type)
      {
        case SDL_ACTIVEEVENT: //if something happens like minimizing or maximixing.
	  if (event.active.state & SDL_APPACTIVE)
	  {
	    if (event.active.gain) //if surface gains activity...
	      active = true;
	    else
	      active = false;     //if surface loses activity....
	  }
	  break;
      
        case SDL_QUIT:
	  exit = true;
	  std::cout << "\nProgram recieved quit request from event handler.\n";
	  break;
	  
	case SDL_KEYDOWN:
	  
	  if (event.key.keysym.sym == SDLK_ESCAPE)
	    exit = true;
	    
	  if (event.key.keysym.sym == SDLK_q)
	    exit = true;
	  	  
	  break;
	 
      }
    }
    
    //keystates are done outside of poll events so we can work as the key is being
    //held down. (aka they don't have to keep tapping it)
    key_state = SDL_GetKeyState(NULL);
    
    //switching to an else if b/c right now if you move at less than 90 degree angles
    //you're get a draw "stain".
    if (key_state[SDLK_LEFT])
    {
      sprite_handler.player_obj.x -= 5;
      sprite_handler.player_obj.ani_wait_count++;
      delta_x = 5;
      delta_y = 0;
      
    } 
    else if (key_state[SDLK_RIGHT])
    {
      sprite_handler.player_obj.x += 5;
      sprite_handler.player_obj.ani_wait_count++;
      delta_x = -5;
      delta_y = 0;

    }
      
    else if (key_state[SDLK_UP])
    {
      sprite_handler.player_obj.y -= 5;
      sprite_handler.player_obj.ani_wait_count++;
      delta_x = 0;
      delta_y = 5;

    }      
    else if (key_state[SDLK_DOWN])
    {
      sprite_handler.player_obj.y += 5;
      sprite_handler.player_obj.ani_wait_count++;
      delta_x = 0;
      delta_y = -5;

    }


    	    
    if (sprite_handler.player_obj.ani_wait_count > 5)
    {
      sprite_handler.player_obj.ani_frame++;
      sprite_handler.player_obj.ani_wait_count = 0;
      
      if (sprite_handler.player_obj.ani_frame > 1)
        sprite_handler.player_obj.ani_frame = 0;
    }


  }
  
  
  std::cout << "Terminating program...\n";
  //sdl quit and such done in the sprite_handler destructor.
  return 0;
}

void move_badguy(Sprite_Handler& handler)
{

  //this is basically a debug function to make the bad guy chase the player.

  if (handler.enemy_obj.x < handler.player_obj.x)  //if players to the right.
    handler.enemy_obj.x += 1;                               //move right.
    
  if (handler.enemy_obj.x > handler.player_obj.x)  //if players to the left
    handler.enemy_obj.x -= 1;                               //move left
  
  if (handler.enemy_obj.y < handler.player_obj.y)  //if players above.
    handler.enemy_obj.y += 1;                               //move up.

  if (handler.enemy_obj.y > handler.player_obj.y)  //if players below.
    handler.enemy_obj.y -= 1;                              //move down.

  handler.enemy_obj.ani_wait_count++;
  
  //switch animation frames every 20 frames.
  if (handler.enemy_obj.ani_wait_count > 20)
  {
    handler.enemy_obj.ani_frame++;
    handler.enemy_obj.ani_wait_count = 0;
    
    if (handler.enemy_obj.ani_frame > 1)
      handler.enemy_obj.ani_frame = 0;
  }
}

