
#include "timer.h"

Timer::Timer()
{
  //initialize the variables
  startTicks = 0;
  pausedTicks = 0;
  
  paused = false;
  started = false;
}

void Timer::start()
{
  //start the timer
  started = true;
  
  //unpause the timer
  paused = false;
  
  //get the current clock time
  startTicks = SDL_GetTicks();
}

void Timer::stop()
{
  //stops the timer
  started = false;
  paused = false;
}

int Timer::get_ticks()
{
  //returns the timers current value or time since it was paused.
  //returns zero if it was never started at all.
  
  if (started)
  {
    if (paused)
      return pausedTicks;
      
    else
      return SDL_GetTicks() - startTicks;
  }
  
  return 0;
}


void Timer::pause()
{
  //pauses the timer if it isn't already paused and it's started.
  if (started && !paused)
  {
    paused = true;
    
    //calculate paused ticks
    pausedTicks = SDL_GetTicks() - startTicks;
  }
}

void Timer::unpause()
{
  //unpause the timer if it's paused & reset the pausedTicks
  if (paused)
  {
    paused = false;
    
    //reset starting ticks
    startTicks = SDL_GetTicks() - pausedTicks;
    
    //reset pausedTicks
    pausedTicks = 0;
  }
}

bool Timer::is_started()
{
  return started;
}

bool Timer::is_paused()
{
  return paused;
}



